/* 
 * Copyright (C) 2006 Cooper Street Innovations Inc.
 *	Charles Eidsness    <charles@cooper-street.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
 * 02110-1301, USA.
 *
 */

        
#ifndef CALC_H
#define CALC_H

#define CALC_MAJOR_VERSION		1
#define CALC_MINOR_VERSION		2

typedef struct _calc calc_;

/* callback: "Get Variable" Call-Back Function Format.
 *		When the parser is parsing an equation and hits a variable,
 *		i.e "4+x" is asks the user / calling function for x.
 * args:	varName - Name of varible that the parser is asking for
 *			private - Reflection of the pointer that was passed to the parser.
 *			type - Type of variable being passed., either 'i', 'v', or 0x0.
 * return:	A pointer to a pointer to the data variable. Return NULL
 *		for an error condition.
 * notes:	Though seemingly inconvenient passing back a pointer to a pointer
 *		makes it possible to manipulate the sets of variables. After the
 *		equation has been created.
 */
typedef double ** (*calcGetVarPtr)(char *varName, char type, void *private);

/* function: Solves the Equation.
 * args:	r - Pointer to a calc structure.
 *			solution - The result will be put in this pointer.
 * return: 0 if it passed, and -1 if it failed.
 */
int calcSolve(calc_ *r, double *solution);

/* function: Solves Derivative of the Equation.
 * args:	r - Pointer to a calc structure.
 *			variable - variable that the derivative is taken w.r.t
 *			type - Type of variabl, either 'i', 'v', or 0x0.
 *			solution - The result will be put in this pointer.
 * return: 0 if it passed, and -1 if it failed.
 */
int calcDiff(calc_ *r, char *variable, char type, double *solution);

/* function: Solves the Zero-Crossing of the Equation.
 * args:	r - Pointer to a calc structure.
 *			solution - The result will be put in this pointer.
 * return: 0 if it passed, and -1 if it failed.
 */
int calcZeroXing(calc_ *r, double *solution);

/* function: Prints out the Library Version Info.
 * return: Top 16 bits hold major version, and bottom 16 bits hold minor.
 */
int calcInfo(void);

/* function: Destroys the calc structure.
 * args:	r - Pointer to a poniter to the calc structure.
 * return: 0 if it passed, and -1 if it failed.
 * notes: Sets the calc structure pointer to NULL in hopes it will raise 
 * 	alarms if some trys to use a destroyed calc.
 */
int calcDestroy(calc_ **r);

/* function: Creates a new calc structure.
 * args:	r - pointer to the pointer to be changed, ie x = ..New(x,...
 *			buffer - String that holds the equation.
 *			getVarFunc - Call-Back function, parser tokenizer calls it when it
 *				needs to get a variable pointer.
 *			private - Pointer that is passed back during the call to getVarFunc.
 *			minDiv - Minimum value for the denominator, used to curcomvent
 *				divide by zero issues.
 * return: Pointer to a new calc struture if it passed, and NULL if it failed.
 */
calc_ * calcNew(calc_ *r, char *buffer, calcGetVarPtr getVarFunc, 
		void *private, double *minDiv);

#endif
