/* 
 * Copyright (C) 2006 Cooper Street Innovations Inc.
 *	Charles Eidsness    <charles@cooper-street.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
 * 02110-1301, USA.
 *
 */

        
#ifndef TOKENIZER_H
#define TOKENIZER_H

/* struc: Basic token structure.
 *		A linked list of these describes the equation for the parser.
 */
typedef struct _token token_;
struct _token {
	int type;			/* Token type, refer to parser.h for types. */
	double **variable;	/* Pointer to a pointer of variabel data, only 
							relavent for TOKEN_VARIABLE types. */
	double constant;	/* holds the value of a constant, only relavent for 
							TOKEN_CONSTANT types. */
	double *minDiv;		/*	Used to pass arguments to operators, like the 
							minimum denominator value */
	token_ *next;
};

/* callback: "Get Variable" Call-Back Function Format.
 *		When the parser is parsing an equation and hits a variable,
 *		i.e "4+x" is asks the user / calling function for x.
 * args:	varName - Name of varible that the parser is asking for
 *			private - Reflection of the pointer that was passed to the parser.
 *			type - Type of variable being passed., either 'i', 'v', or 0x0.
 * return:	A pointer to a pointer to the data variable. Return NULL
 *		for an error condition.
 * notes:	Though seemingly inconvenient passing back a pointer to a pointer
 *		makes it possible to manipulate the sets of variables. After the
 *		equation has been created.
 *			This callbakc is equivalent to the calcGetVarPtr call-back in
 *		the calc.h header file.
 */
typedef double ** (*tokenizerGetVarPtr)(char *varName, char type, 
		void *private);

/* function: Destroys all of the token structures attached to a linked list.
 * args:	r - Pointer to a pomiter to the token structure.
 * return: 0 if it passed, and -1 if it failed.
 * notes: Sets the token structure pointer to NULL in hopes it will raise 
 * 	alarms if some trys to use a destroyed token.
 */
int tokenizerDestroyAll(token_ **r);

/* function: Creates a new token structure linked list.
 *		The equation is tokenized by an re2c based lexer.
 * args:	r - pointer to the pointer to be changed, ie x = ..New(x,...
 * 			buffer - String that holds the equation.
 *			getVarFunc - Call-Back function, parser tokenizer calls it when it
 *				needs to get a variable pointer.
 *			private - Pointer that is passed back during the call to getVarFunc.
 *			minDiv - Minimum value for the denominator, used to curcomvent
 *				divide by zero issues.
 * return: Pointer to a token struture if it passed, and NULL if it failed.
 */
token_ * tokenizerNew(token_ *r, char *buffer, tokenizerGetVarPtr getVarFunc, 
		void *private, double *minDiv);

#endif
