/* 
 * Copyright (C) 2006 Cooper Street Innovations Inc.
 *	Charles Eidsness    <charles@cooper-street.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
 * 02110-1301, USA.
 *
 */

#ifndef VARIABLE_H
#define VARIABLE_H

/* struc: Basic variable structure.
 *		A linked list that holds all of the unique varibales in the 
 *		equation and pointers to the varibale value.
 */
typedef struct _variable variable_;
struct _variable {
	char *name; 	/* String containg name of the variable. (local copy) */
	char type;		/* Type of variable. */
	double **data;	/* Pointer to a pointer to the varibale's data. */
	variable_ *next;
};

/* function: Creates a new variable structure and attaches it to the
 *		linked list of variables.
 * args:	r - Pointer to the head of the linked list.
 *			name - String, name of the variable.
 *			type - Type of variable.
 * return: pointer to the new variable if it passed, and NULL if it failed.
 */
variable_ * variableAttachNew(variable_ *r, char *name, char type);

/* function: Finds and retuns a variable from the linked list.
 * args:	r - Pointer to the head of the linked list.
 *			name - String, name of the variable.
 *			type - Type of variable.
 * return: pointer to the pointer to the variable data if it passed, 
 *			and NULL if it failed.
 */
double ** variableFindData(variable_ *r, char *name, char type);

/* function: Destroys all of the variable structures attached to a linked list.
 * args:	r - Pointer to a pomiter to the variable structure.
 * return: 0 if it passed, and -1 if it failed.
 * notes: Sets the variable structure pointer to NULL in hopes it will raise 
 * 	alarms if some trys to use a destroyed variable.
 */
int variableDestroyAll(variable_ **r);

/* function: Creates a new variable structure linked list.
 * args:	r - pointer to the pointer to be changed, ie x = ..New(x,...
 * return: Pointer to a variable struture if it passed, and NULL if it failed.
 */
variable_ * variableNew(variable_ *r);

#endif
