/* 
 * Copyright (C) 2006 Cooper Street Innovations Inc.
 *	Charles Eidsness    <charles@cooper-street.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
 * 02110-1301, USA.
 *
 */

#include <log.h>
#include "command_internal.h"

static char * type = "Operating Point Analysis";

struct _commandPrivate {
	int nothing;
};

static int execute(command_ *r, state_ *state)
{
	commandPrivate_ *p;
	
	ReturnErrIf(r == NULL);
	p = r->private;
	ReturnErrIf(p == NULL);
	
	/* Load Matricies */
	ReturnErrIf(deviceDBLoad(r->devices, state));
	
	/* Solve Matracies */
	ReturnErrIf(commandSolve(r, state, stateGetITL1(state)) < 0);
	
	/* Store Data (time is 0 and no break-point */
	ReturnErrIf(matrixRecord(r->matrix, 0.0, 0));
	
	return 0;
}

int commandConfigOpPoint(command_ *r)
{
	commandPrivate_ *p;
	
	ReturnErrIf(r == NULL);
	ReturnErrIf(r->private != NULL);
	
	Debug("Configuring %s %p", type, r);
	
	r->type = type;
	r->execute = execute;
	
	r->private =  calloc(1, sizeof(commandPrivate_));
	ReturnErrIf(r->private == NULL);
	p = r->private;
	
	return 0;
}
