/* 
 * Copyright (C) 2006 Cooper Street Innovations Inc.
 *	Charles Eidsness    <charles@cooper-street.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
 * 02110-1301, USA.
 *
 */

#include <log.h>
#include "ei.h"
#include "deck.h"

struct _ei {
	deck_ *deck;
};

int eiWriteRaw(ei_ *r, FILE *outFile)
{
	ReturnErrIf(r == NULL);
	ReturnErrIf(deckWriteRaw(r->deck, outFile));
	return 0;
}

int eiProcess(ei_ *r)
{
	ReturnErrIf(r == NULL);
	ReturnErrIf(deckProcess(r->deck));
	return 0;
}

int eiInfo(void)
{
	Info("Ei Spice Engine %i.%i", EI_MAJOR_VERSION, EI_MINOR_VERSION);
	Info("Compiled " __DATE__ " at " __TIME__);
	Info("(c) 2006 Cooper Street Innovations Inc.");
	return ((EI_MAJOR_VERSION << 16) + EI_MINOR_VERSION);
}

int eiDestroy(ei_ **r)
{
	ReturnErrIf(r == NULL);
	ReturnErrIf(*r == NULL);
	Debug("Destroying ei %p", *r);
	
	deckDestroy(&(*r)->deck);
	
	free(*r);
	*r = NULL;
	return 0;
}

ei_ * eiNew(ei_ *r, char *filename)
{
	ReturnNULLIf(NULL != 0);
	ReturnNULLIf(r != NULL);
	ReturnNULLIf(filename == NULL);
	r = calloc(1, sizeof(ei_));
	ReturnNULLIf(r == NULL);
	
	Debug("Creating ei %p", r);
	
	r->deck = deckNew(r->deck, filename);
	ReturnNULLAndFreeIf(r, r->deck == NULL);
	
	return r;
}
