/* 
 * Copyright (C) 2006 Cooper Street Innovations Inc.
 *	Charles Eidsness    <charles@cooper-street.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
 * 02110-1301, USA.
 *
 */

#ifndef DEVICE_H
#define DEVICE_H

#include "matrix.h"
#include "row.h"
#include "state.h"

#define DEVICE_MAX_PINS 4

typedef struct _device device_;
typedef struct _deviceDB deviceDB_;

typedef enum {
	LIN_ERR = -1,
	LIN_LINEAR = 0,
	LIN_NOT_LINEAR = 1,
} deviceLinear;

typedef enum {
	NI_ERR = -1,
	NI_BE = 0,
	NI_TRAP = 1,
} deviceNIType;

int deviceDBLoad(deviceDB_ *r, state_ *state);
deviceLinear deviceDBLinearize(deviceDB_ *r, state_ *state);
int deviceDBStep(deviceDB_ *r, state_ *state, double time, deviceNIType niType);
double deviceDBCalcNextStep(deviceDB_ *r, state_ *state, deviceNIType niType);
double deviceDBCalcNextBreak(deviceDB_ *r, state_ *state, double lastBreak, 
		double time);

int deviceDBAttach(deviceDB_ *r, device_ *device);
int deviceDBDestroy(deviceDB_ **r);
deviceDB_ * deviceDBNew(deviceDB_ *r);

typedef enum {
	SOURCE_NONE,
	SOURCE_PULSE,
	SOURCE_SIN,
	SOURCE_EXP,
	SOURCE_PWL,
	SOURCE_SFFM,
} srcType_;

int deviceConfigSource(device_ *r, double dc, srcType_ srcType, 
		double *srcData, int srcLength, char type);
int deviceConfigVCCS(device_ *r, double gain);
int deviceConfigVCVS(device_ *r, double gain);
int deviceConfigCCCS(device_ *r, double gain);
int deviceConfigCCVS(device_ *r, double gain);
int deviceConfigNonlinearV(device_ *r, char *equation);
int deviceConfigNonlinearI(device_ *r, char *equation);
int deviceConfigResistor(device_ *r, double R);
int deviceConfigInductor(device_ *r, double L, double ic);
int deviceConfigCapacitor(device_ *r, double C, double ic);
int deviceConfigTLine(device_ *r, double Z0, double Td);

int deviceDestroy(device_ **r);
device_ * deviceNew(device_ *r, matrix_ *matrix, char *refdes, row_ *pin[], 
		int numPins);


#endif
